//+------------------------------------------------------------------+
//|                                            TickStochastic_v1.mq4 |
//|                             Copyright  2007-08, TrendLaboratory |
//|                          Many Thanks to Rosh for Ticks indicator |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
#property copyright "Copyright  2007-08, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"


#property indicator_separate_window
#property indicator_buffers   2
#property indicator_color1    DodgerBlue
#property indicator_color2    Tomato
#property indicator_style2    2
#property indicator_maximum   100
#property indicator_minimum   0
#property indicator_level1    30
#property indicator_level2    70
//---- external variables
extern int     Length      =    14; //Length(Period) of Channel
extern int     Smooth      =     8;
extern int     Signal      =     8;
extern int     MA_Mode     =     0; //MA Mode: 0-SMA,1-EMA,2-Wilder(SMMA),3-LWMA 
extern int     UseDelimiter=     1; //Delimiter Mode: 0-off,1-on
extern color   DelimColor  =  Gray; //Color of Bars Delimiter 
extern int     MaxTicks    =   70; //Max Number of ticks   
//---- indicator buffers
double Sto[];
double Sig[];
double WPR[];
double Ticks[];
//---- global variables
int      tickCounter=0;
int      delimeterCounter;
datetime pTime;
string   short_name, setup;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
//---- indicators
   IndicatorBuffers(4);
   SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(0, Sto);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexBuffer(1, Sig);
   SetIndexBuffer(2, WPR);
   SetIndexBuffer(3, Ticks);
   
   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS)+2);
   short_name="TickStochastic_v1("+Length+","+Smooth+","+Signal+","+UseDelimiter+")";
   IndicatorShortName(short_name);
   SetIndexLabel(0,"Sto");
   SetIndexLabel(1,"Signal");
      
   SetIndexEmptyValue(0,0.0);
   SetIndexEmptyValue(1,0.0);
         
   setup = short_name+": ";
   pTime = Time[0];
//----
   return(0);
}
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
//----
   ObjDel(setup);
//----
   return(0);
}

void SetDelimeter()
{
  //----
  string delimeterName = setup + TimeToStr(Time[0]);
  
  int phandle=WindowFind(short_name);
  if(phandle > -1)
  {
    if(!ObjectCreate(delimeterName,OBJ_VLINE,phandle,Time[0],0))
    {
      Print("Error delimiter:",GetLastError());
    }
    else 
    {
      ObjectSet(delimeterName,OBJPROP_COLOR,DelimColor);
      ObjectSet(delimeterName,OBJPROP_STYLE,STYLE_DOT);
    }
  }
}

void ShiftArray()
{
   
   for(int cnt=tickCounter-1; cnt > 0; cnt--)
   {
   Ticks[cnt] = Ticks[cnt-1];
   Sto [cnt] = Sto[cnt-1];
   Sig [cnt] = Sig[cnt-1];
   WPR [cnt] = WPR[cnt-1];
   }
   
   if (UseDelimiter!=0)
   {
      for (int j=0;j<ObjectsTotal();j++)
      {
      int NumStr = StringFind(ObjectName(j),setup,0);
      
         if (NumStr == 0)
         {
         datetime Time1=ObjectGet(ObjectName(j),OBJPROP_TIME1);
         int BarTime1=iBarShift(NULL,0,Time1);
         Time1 = Time[BarTime1+1];
         ObjectSet(ObjectName(j),OBJPROP_TIME1,Time1); 
            if( BarTime1+1 > MaxTicks )
            {
               if (!ObjectDelete(ObjectName(j)))
               {
               int _GetLastError = GetLastError();
               Print("ObjectDelete: ",ObjectName(j)," Error #", _GetLastError );
               }
            }
         }       
      }
   }      
}

bool isNewBar()
{
   bool res=false;
   if (Time[0]!=pTime)
   {
   res=true;
   pTime=Time[0];
   }   

   return(res);
}

bool ObjDel(string name)
{
   int _GetLastError = 0;
     
   while(ObjFind(name,0,0) > 0)
   {
      int obtotal = ObjectsTotal();
      for (int i = 0; i < obtotal;i++)
      {
         if (StringFind(ObjectName(i),name,0) >= 0)
         {
            if (!ObjectDelete(ObjectName(i)))
            {
            _GetLastError = GetLastError();
            Print( "ObjectDelete( \"",ObjectName(i),"\" ) - Error #", _GetLastError );
            }
         }
      }
   }      
   if(_GetLastError > 0) return(false);
   else
   return (true);
}
//-----
int ObjFind(string name,int start, int num)
{
   int cnt = 0;
   
   for (int i = 0; i < ObjectsTotal();i++)
      if (StringFind(ObjectName(i),name,start) == num) cnt+=1;
   
   return(cnt);
}         
  
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   if(tickCounter == 0)
   {
      for(int i=Bars-1;i>=0;i--)
      {
      Ticks[i]=0.0;
      Sto[i]=0.0;
      Sig[i]=0.0;
      WPR[i]=0.0;
      }
   }
   
   tickCounter++;
   
   if(tickCounter >= MaxTicks) 
   {
   tickCounter = MaxTicks; 
   Ticks[tickCounter]=0.0;
   Sto[tickCounter]=0.0;
   Sig[tickCounter]=0.0;
   WPR[tickCounter]=0.0;
   }
  
   if (isNewBar()){if(UseDelimiter == 1) SetDelimeter();}
   else
   ShiftArray();
      
   Ticks[0]=Bid;
   
   if(tickCounter>=Length)
   {
   double Up = TickHighest(Ticks, Length); 
   double Dn = TickLowest (Ticks, Length); 
   if((Up-Dn) > 0) WPR[0] = 100*(Ticks[0] - Dn)/(Up - Dn); else WPR[0] = 0; 
   
      if(MA_Mode == 0)
      {
      if(tickCounter>=Length + Smooth) Sto[0] = TickSMA(WPR,Smooth);          
      if(tickCounter>=Length + Smooth+Signal) Sig[0] = TickSMA(Sto,Signal);
      }   
      else
      if(MA_Mode == 1)
      {
      if(tickCounter == Length + Smooth) Sto[0] = TickSMA(WPR,Smooth);  
      else if(tickCounter > Length + Smooth) Sto[0] = TickEMA(WPR,Sto,Smooth);
      if(tickCounter == Length + Smooth + Signal) Sig[0] = TickSMA(Sto,Signal);  
      else if(tickCounter > Length + Smooth + Signal) Sig[0] = TickEMA(Sto,Sig,Signal);
      }
      else
      if(MA_Mode == 2)
      {
      if(tickCounter == Length + 2*Smooth-1) Sto[0] = TickSMA(WPR,2*Smooth-1);  
      else if(tickCounter > Length + 2*Smooth-1) Sto[0] = TickEMA(WPR,Sto,2*Smooth-1);
      if(tickCounter == Length + 2*(Smooth + Signal)-2) Sig[0] = TickSMA(Sto,2*Signal-1);  
      else if(tickCounter > Length + 2*(Smooth + Signal)-2) Sig[0] = TickEMA(Sto,Sig,2*Signal-1);
      }
      else
      if(MA_Mode == 3)
      {
      if(tickCounter>=Length + Smooth) Sto[0] = TickLWMA(WPR,Smooth);          
      if(tickCounter>=Length + Smooth+Signal) Sig[0] = TickLWMA(Sto,Signal);
      }   
   }   
//----
   return(0);
}
//+------------------------------------------------------------------+
double TickLowest(double array[],int per)
{
   double lowest = 1000000;
   for(int i = 0;i < per;i++) lowest = MathMin(lowest,array[i]);
   return(lowest);
}                

double TickHighest(double array[],int per)
{
   double highest = 0;
   for(int i = 0;i < per;i++) highest = MathMax(highest,array[i]);
   return(highest);
}                

double TickSMA(double array[],int per)
{
   double Sum = 0;
   for(int i = 0;i < per;i++) Sum += array[i];
   return(Sum/per);
}                

double TickEMA(double array1[],double array2[],int per)
{
   return(array2[1] + 2.0/(1+per)*(array1[0] - array2[1]));
}

double TickLWMA(double array[],int per)
{
   double Sum = 0;
   double Weight = 0;
   
      for(int i = 0;i < per;i++)
      { 
      Weight+= (per - i);
      Sum += array[i]*(per - i);
      }
   if(Weight>0) double lwma = Sum/Weight;
   else lwma = 0; 
   return(lwma);
}                          